diff a/hat/backends/ffi/opencl/src/main/native/cpp/opencl_backend_program.cpp b/hat/backends/ffi/opencl/src/main/native/cpp/opencl_backend_program.cpp
--- /dev/null
+++ b/hat/backends/ffi/opencl/src/main/native/cpp/opencl_backend_program.cpp
@@ -0,0 +1,52 @@
+/*
+ * Copyright (c) 2024, Oracle and/or its affiliates. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
+ *
+ * This code is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 only, as
+ * published by the Free Software Foundation.  Oracle designates this
+ * particular file as subject to the "Classpath" exception as provided
+ * by Oracle in the LICENSE file that accompanied this code.
+ *
+ * This code is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * version 2 for more details (a copy is included in the LICENSE file that
+ * accompanied this code).
+ *
+ * You should have received a copy of the GNU General Public License version
+ * 2 along with this work; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
+ */
+
+#include "opencl_backend.h"
+
+OpenCLBackend::OpenCLProgram::OpenCLProgram(Backend *backend,  char *src, char *log, bool ok, cl_program program)
+    : CompilationUnit(backend, src,log, ok), program(program) {
+}
+
+OpenCLBackend::OpenCLProgram::~OpenCLProgram() {
+    clReleaseProgram(program);
+}
+
+ Backend::CompilationUnit::Kernel *OpenCLBackend::OpenCLProgram::getKernel(int nameLen, char *name) {
+    cl_int status;
+    cl_kernel kernel = clCreateKernel(program, name, &status);
+    if (status != CL_SUCCESS){
+       std::cerr << "Failed to get kernel "<<name<<" "<<errorMsg(status)<<std::endl;
+    }
+    return new OpenCLKernel(this,name, kernel);
+}
+OpenCLBackend::OpenCLProgram::OpenCLKernel *OpenCLBackend::OpenCLProgram::getOpenCLKernel(int len, char *name) {
+   return dynamic_cast<OpenCLKernel *>(getKernel(len, name));
+}
+
+OpenCLBackend::OpenCLProgram::OpenCLKernel *OpenCLBackend::OpenCLProgram::getOpenCLKernel(char *name) {
+   return getOpenCLKernel(strlen(name), name);
+}
+
+
